#!/bin/sh
#############################################################################
# 
# PowerChute Business Edition v.7.0.x JRE Update
# Copyright (C)2006 American Power Conversion, Inc. All Rights Reserved.
#
# Revision:
#   1.0.0.7 : 30Jan2006 
#
#############################################################################

#############################################################################
# Grobal Constants
#############################################################################
SOLARIS="Solaris"
LINUX="Linux"

SOLARIS_JRE_TAR="jre142_10sol.tar"
SOLARIS_STARTUP="/etc/rc2.d/S99PBEAgent"
SOLARIS_STOP="/etc/rc0.d/K99PBEAgent"

LINUX_JRE_TAR="jre142_10lnx.tar"
LINUX_STARTUP="/etc/init.d/PBEAgent"

BIN_STARTUP="/bin/startup"
UNINSTALLER_FILE="uninstallPowerChuteBusinessEditionAgent"

JREJRE14="jre1.4"
JRE14="1.4"
JRE14_14="jre1.4/1.4"
JRE14_JRE14="jre1.4/jre1.4"
JRE142="1.4.2"
JRE142_06="1.4.2_06"
JRE142_10="1.4.2_10"
TMP_FILE="/tmp/_tmp_pbeagent"

############################################################################
# Grobal Variables
#############################################################################
STARTUP=""
JRE_TAR=""
JRE_ZIP=""

JRE_ROOT=""
JRE_OLD=""
JRE_NEW=""
JRE_VER=""

OLD_VER=""
NEW_VER=""
AGENT_PATH=""

SILENT_MODE=0
OS_TYPE=""

#############################################################################
# Functions
#############################################################################
Echo() {
    string="$1"
    if [ $SILENT_MODE = 0 ]
    then
        echo "$string"
    fi
}

PrintUsage() {
    echo "Usage:"
    echo "  $0 [-h|-H] : Print help."
    echo "  $0 [-s|-S] : Silent install."
    exit 1
}

IsSilentMode() {
    if [ -z "$1" ]
    then
        SILENT_MODE=0
    elif [ "$1" = "-s" -o "$1" = "-S" ]
    then 
        SILENT_MODE=1
    else
        PrintUsage
    fi
}


CheckOS() {
    os_name=`uname -a|tr "[a-z]" "[A-Z]"|grep LINUX`
    if [ ! -z "$os_name" ]
    then
        OS_TYPE=$LINUX
    else 
        os_name=`uname -a|tr "[a-z]" "[A-Z]"|grep SOLARIS`
        if [ ! -z "$os_name" ]
        then
            OS_TYPE=$SOLARIS
        else
            os_name=`uname -a|tr "[a-z]" "[A-Z]"|grep SUNOS`
            if [ ! -z "$os_name" ]
            then
                OS_TYPE=$SOLARIS
            fi
        fi
        cpu=`uname -a|tr "[a-z]" "[A-Z]"|grep SPARC`
        if [ "$OS_TYPE" = "$SOLARIS" -a -z "$cpu" ]
        then
            Echo "It's not Solaris(SPARC)."
            exit 1
        fi
    fi
    if [ -z "$OS_TYPE" ]
    then
        Echo "Unknown OS type."
        exit 1
    else
        Echo "OS type: $OS_TYPE"
    fi
}

Initialize() {
    if [ "$OS_TYPE" = "$LINUX" ]
    then
        STARTUP=$LINUX_STARTUP
        JRE_TAR=$LINUX_JRE_TAR
    elif [ "$OS_TYPE" = "$SOLARIS" ]
    then
        STARTUP=$SOLARIS_STARTUP
        JRE_TAR=$SOLARIS_JRE_TAR
    else
        Echo "Unknown OS type."
        exit 1
    fi
    JRE_ZIP="$JRE_TAR.gz"
}

IsRootUser() {
    root_str=`id | grep root`
    if [ -z "$root_str" ]
    then
        Echo "$0 must be run with root privileges!"
        exit 1
    fi
}

IsPCBEInstalled() {
    if [ ! -f $STARTUP ]
    then
        Echo "PCBE Agent is not installed."
        exit 1
    fi
}

IsPatchInstalled() {
    buf=`echo $JRE_OLD | grep $JRE142_10`
    if [ ! -z "$buf" ]
    then
        Echo "PCBE v.7.0.x JRE Update is already installed."
        exit 1
    fi
}

GetJrePath() {
    # Find JRE path
    search_str="\-Dpicard\.main\.thread"
    if [ "$OS_TYPE" = "$SOLARIS" ]
    then
        remove_str_1="^.*nohup."
        remove_str_2="\/bin\/java.*\-Dpicard\.main\.thread.*$"
    else
        remove_str_1="^\t*"
        remove_str_2="\/bin\/java.\-Dpicard\.main\.thread.*$"
    fi

    buf=`grep $search_str $STARTUP`
    JRE_OLD=`echo $buf | sed -e s/$remove_str_1//g -e s/$remove_str_2//g`
    if [ -z "$JRE_OLD" ]
    then
        Echo "Can't find JRE"
        exit 1
    else
        Echo "PCBE JRE dir is \"$JRE_OLD\""
    fi
    
    # Update Patch already installed?
    IsPatchInstalled

    jre142_06="\/1\.4\.2_06$"
    jre142="\/1\.4\.2$"
    jre14_14="\/jre1\.4\/1\.4$"
    jre14_jre14="\/jre1\.4\/jre1\.4$"
    jre14="\/1\.4$"
    jrejre14="\/jre1\.4$"
    buf=`echo $JRE_OLD | grep $jre142_06`
    if [ -z "$buf" ]
    then
        buf=`echo $JRE_OLD | grep $jre142`
        if [ -z "$buf" ]
        then
            buf=`echo $JRE_OLD | grep $jre14_14`
            if [ -z "$buf" ]
            then
                buf=`echo $JRE_OLD | grep $jre14_jre14`
                if [ -z "$buf" ]
                then
                    buf=`echo $JRE_OLD | grep $jre14`
                    if [ -z "$buf" ]
                    then
                        buf=`echo $JRE_OLD | grep $jrejre14`
                        if [ -z "$buf" ]
                        then
                            Echo "Can't find JRE 1.4.x for PCBE Agent"
                            exit 1
                        else
                            JRE_VER=$JREJRE14
                            JRE_ROOT=`echo $buf|sed s/$jrejre14//`  
                        fi
                    else 
                        JRE_VER=$JRE14
                        JRE_ROOT=`echo $buf|sed s/$jre14//`  
                    fi
                else
                   JRE_VER=$JRE14_JRE14
                   JRE_ROOT=`echo $buf|sed s/$jre14_jre14//`  
                fi
            else 
                JRE_VER=$JRE14_14
                JRE_ROOT=`echo $buf|sed s/$jre14_14//`  
            fi
        else 
            JRE_VER=$JRE142
            JRE_ROOT=`echo $buf|sed s/$jre142//`  
        fi
    else 
        JRE_VER=$JRE142_06
        JRE_ROOT=`echo $buf|sed s/$jre142_06//`  
    fi
    JRE_NEW="$JRE_ROOT/$JRE142_10"
    Echo "PCBE JRE Version: $JRE_VER"
    Echo "JRE Root Path   : $JRE_ROOT"
    Echo "Current JRE Path: $JRE_OLD"
    Echo "New JRE Path    : $JRE_NEW"
}

ConfirmInstallation() {
    if [ $SILENT_MODE = 1 ]
    then
        return
    fi
    response=1
    while [ $response = 1 ]
    do
        ynq=
        echo ""
        echo "Are you sure to install? [y/n]"
        read ynq
        case "$ynq" in
        [Yy]*)
            response=0
            ;;
        [Nn]*)
            echo "Installation canceled."
            exit 1
            ;;
        *)
            echo "Invalid response."
            response=1
            ;;
        esac
    done
    echo ""
}

IsJreInstalled() {
    if [ -f $JRE_ROOT/$JRE142_10 -o -d $JRE_ROOT/$JRE142_10 ]
    then
        rval=1
    else
        rval=0
    fi
    return $rval
}

InstallJre() {
    Echo "Installing JRE..."
    cp $JRE_ZIP $JRE_ROOT
    cd $JRE_ROOT
    if [ ! -f $JRE_ZIP ]
    then
        Echo "Can't find $JRE_ZIP"
        exit 1
    fi
    if [ $SILENT_MODE = 0 ]
    then
        gunzip $JRE_ZIP
        tar -xvf $JRE_TAR
    else 
        gunzip $JRE_ZIP 1>/dev/null 2>/dev/null
        tar -xvf $JRE_TAR 1>/dev/null 2>/dev/null
    fi
    if [ $? != 0 ]
    then
        echo "Failed to install JRE. Please check available disk space."
        rm -rf $JRE_ROOT/$JRE_TAR
        rm -rf $JRE_ROOT/$JRE_ZIP
        rm -rf $JRE_ROOT/$JRE142_10
        exit 1
    fi
    rm -rf $JRE_ROOT/$JRE_TAR

    # Generate jre Class Sharing File 
    $JRE_NEW/bin/java -Xshare:dump 1>/dev/null 2>/dev/null
}

GetPCBEAgentPath() {
    # Find Agent path
    remove_str_1=".*LD_LIBRARY_PATH:"
    if [ "$OS_TYPE" = "$SOLARIS" ]
    then
        remove_str_2="\/lib\/solaris"
    else
        remove_str_2="\/lib\/linux"
    fi

    grep $remove_str_2$ $STARTUP > $TMP_FILE
    AGENT_PATH=`grep $remove_str_2$ $STARTUP | sed -e s/$remove_str_1//g -e s/$remove_str_2//g `
    Echo "PCBE Agent dir is $AGENT_PATH"
}

UpdateStartUp() {
    Echo "Updating startup files..."
    if [ $JRE_VER = "$JRE142_06" ]
    then 
        OLD_VER="\/1\.4\.2_06"
    elif [ $JRE_VER = "$JRE142" ]
    then 
        OLD_VER="\/1\.4\.2"
    elif [ $JRE_VER = "$JRE14_14" ]
    then 
        OLD_VER="\/jre1\.4\/1\.4"
    elif [ $JRE_VER = "$JRE14_JRE14" ]
    then 
        OLD_VER="\/jre1\.4\/jre1\.4"
    elif [ $JRE_VER = "$JREJRE14" ]
    then 
        OLD_VER="\/jre1\.4"
    else
        OLD_VER="\/1\.4"
    fi

    NEW_VER="\/$JRE142_10"
    if [ -z "$OLD_VER" -o -z "$NEW_VER" ]
    then
        Echo "Can't find jre version name."
        exit 1
    fi

    sed s/$OLD_VER/$NEW_VER/g $STARTUP > $TMP_FILE
    cp $TMP_FILE $STARTUP
 
    # Update local startup 
    startup="$AGENT_PATH$BIN_STARTUP"
    if [ -f $startup ]
    then
        Echo "Copy $startup" 
        cp $STARTUP $startup
    fi

    # Update local stop script 
    if [ "$OS_TYPE" = "$SOLARIS" -a -f $SOLARIS_STOP ]
    then
        Echo "Copy $SOLARIS_STOP" 
        cp $STARTUP $SOLARIS_STOP
    fi
}

UpdateUninstall() {
    if [ -z "$OLD_VER" -o -z "$NEW_VER" ]
    then
        Echo "Can't find jre version name."
        exit 1
    fi

    # Set uninstall script
    uninstall_script="$AGENT_PATH/$UNINSTALLER_FILE"
    if [ -f $uninstall_script ]
    then
        # Update uninstall script
        Echo "Updating uninstall files..."
        sed s/$OLD_VER/$NEW_VER/g $uninstall_script > $TMP_FILE
        cp $TMP_FILE $uninstall_script
        
        # Copy JExpressUninstaller.jar
        cp $JRE_OLD/JExpressUninstaller.jar $JRE_NEW
    fi
}

#############################################################################
# Main routine
#############################################################################
PWD=`pwd`

# Check silent install
IsSilentMode $1

Echo "* "
Echo "* Install PowerChute Business Edition v.7.0.x JRE Update"
Echo "* "

# Check root account
IsRootUser

# Check OS type
CheckOS
Initialize

# Check PCBE Agent
IsPCBEInstalled

# Find JRE path
GetJrePath

# Confirm Installation
ConfirmInstallation

# Stop PBEAgent daemon
$STARTUP stop

# Install JRE
IsJreInstalled
if [ $? = 0 ]
then
    InstallJre
else
    Echo "JRE1.4.2_10 is already installed. Use installed JRE."
fi

# Get PCBE Agent installed dir
GetPCBEAgentPath

# Update startup script
UpdateStartUp

# Copy JExpress file
UpdateUninstall

# Start PBEAgent daemon
$STARTUP start

rm -rf $TMP_FILE
Echo "Installation completed."
cd $PWD

exit 0

### EOF ###
